/*
  With an statistic object it is possible to measure a value over time and calculate average, min and max.
  Besides that it is possible to draw a plot of the measurements by calling the function interface().
  Example:

   Statistic::Ptr statistic = make_statistic("Statistic Name", 128);
   statistic->add_sample(42.0f);
   statistic->interface();
*/

#pragma once
#include <memory>
#include <string>
#include <vector>
#include <cstdint>

class Statistic
{
public:
    typedef std::shared_ptr<Statistic> Ptr;

private:
    std::string name;
    std::vector<float> samples;

    uint32_t current_index = 0;
    uint32_t valid_index = 0;

public:
    Statistic(const std::string& name, uint32_t sample_count);

    void interface();

    void add_sample(float sample);

    float get_average() const;
    float get_min() const;
    float get_max() const;

    const std::string& get_name() const;
    const std::vector<float>& get_samples() const;
};

Statistic::Ptr make_statistic(const std::string& name, uint32_t sample_count);